*	-------------------------- S A M P L E   P R O G R A M   C O D E   S T A T A --------------------------------------------
/*                               
	Note: The sample program code serves as an orientation for writing your program code. The objective is to 
    generate an output file that is helping the output check for confidentiality in the best possible way. 
    Please note: This sample program code does not provide statistic-specific support on the processing and 
    analysis of your requested data.

	In advance, here are some basic aspects for your work with Stata in the RDC context:
	
	- Please delete temporary and/or working datasets that you do not need anymore at the end of your programs.
	
	- Please define your work environment (e.g. safe centre, remote execution) in the first part of your do-files
	  and introduce your (possibly multiple) datasets.
	
	- Please consider the "Basic requirements concerning the program code" and the "List of criteria on the admission
	  of output" that you find in the brochure "Regulations on the analysis of microdata".
	
	- Please create a master program if you are using multiple do-files for your analyses.

	Please also note that the online access to ado-files is not possible via the RDC. Please provide us with a version
	of the ado-file that we can save locally if you want to use ado-files. Ado-Files are only allowed after they were
	checked by the RDC staff.
*/                                                                                                                      



*	-------------------------------------------------------------------------------------------------------------------
*	----
*	----	Basic settings
*	----
*	-------------------------------------------------------------------------------------------------------------------

clear all
set more off
* further basic settings, if needed 



*	-------------------------------------------------------------------------------------------------------------------
*	----
*	----	Part I: Define work environment
*	----
*	-------------------------------------------------------------------------------------------------------------------

/*
At the RDC, we generally differ between three work environments:
0. personal work environment at the office/at home
1. remote execution
2. safe centre

There are RDC products that only run in one work environment, e.g. pure safe centre projects.
However, there are also RDC projects that run in several work environments. There are, for example, datasets that 
can't be provided in full at the safe centre (economic data, tax data). Users have only limited access to the 
data at the safe centre but they can use it to write program codes. Those are then applied to the original data by 
the RDC staff. This makes an adaptation of the path specification necessary.

To make those basic settings clear, we kindly ask you to define your work environment and to put in the according
paths.

Please differ between the following three work environments and adjust the paths for all relevant work environments.
Please delete paths that are not needed and add missing ones.

*	0: personal workspace (e.g. data structure file)
*	1: remote execution (original dataset)
*	2: safe centre (pseudonymised original data, possibly without Bavaria)
*/



* global RDC 0
global RDC 1 // Arbeitsumgebung bestimmen, hier auf KDFV gestellt (siehe oben)
* global RDC 2



*	Note on the definition on the work environment:
*	Please do not change the names of the globals for the definition of the destination folder. Should you need 
*	several destination folders for one type of results, please name them according to the following pattern: 
*	global outputpfad-1, global outputpfad-2, ...
		
*	Work environment 0: Path specification for working at the personal workplace (e.g. data structure file)
*	-------------------------------------------------------------------------------------------------------------------

if $FDZ==0 {
	global datenpfad    "<personal workspace: data>" /* contains, for example, the data structure file */			   
	global outputpfad   "<personal workspace: results>" /* all results (incl. log files) are to be saved here */
	global neudatenpfad "<personal workspace: work files>" /* all newly created datasets are to be saved here */
	
	global dateiname "insert name of dataset.dta" /* e.g. name of the data structure file */
	global outputname "insert name of log file" /* insert name of the log file */
	
	adopath         + "<personal workplace ado files>" /* Here, all necessary ado files are saved locally */
}


*	Work environment 1: Path specifications for working via remote execution (note: you are given all relevant paths via e-mail at the begin of your project)
*	-------------------------------------------------------------------------------------------------------------------

if $FDZ==1 {
	global datenpfad    "Q:\Projektxxxx\Daten" /* contains the original data */			   
	global outputpfad   "Q:\Projektxxxx\xxx" /* all results (incl. log files) are to be saved here */
	global neudatenpfad "Q:\Projektxxxx\xxx" /* all work files are to be saved here */
	
	global dateiname "insert name of dataset.dta" /* insert name of the original dataset (see e-mail) */
	global outputname "insert name of log file" /* insert name of the log file */

	adopath         + "Q:\stata\ado" /* Here, all necessary ado files are saved locally */
}


*	Work environment 2: Path specifications for working at the safe centre(_formal)
*	-------------------------------------------------------------------------------------------------------------------

if $FDZ==2 {
	global datenpfad    "R:\Projektxxxx\Daten" /* contains the original data */			   
	global outputpfad   "R:\Projektxxxx\xxx" /* all results (incl. log files) are to be saved here */
	global neudatenpfad "R:\Projektxxxx\xxx" /* all work files are to be saved here */
	
	global dateiname "insert name of dataset.dta" /* pseudonymised original data, possibly without Bavaria */
	global outputname "insert name of log file"
	
	adopath         + "R:\stata\ado" /* Here, all necessary ado files are saved locally */

}



*	-------------------------------------------------------------------------------------------------------------------
*	----
*	----	Part II: Processing of the data
*	----
*	-------------------------------------------------------------------------------------------------------------------

*	Start log
capture log close
log using "$outputpfad/${outputname}.log", replace





*	-------------------------------------------------------------------------------------------------------------------
/*
	Project title:			<Women and employment in Germany>       
	Data base:		 		<Example Microcensus 2012>
	
	Name of program code:	<syntax_name.do> // if applicable, please also state the version of the program code
	created: 				<Date> 
	by: 					<Name> 
	E-mail: 				<E-mail address> 
	Phone: 					<phone number> 
	
	Name of output file: 	<output_name.log>
	
	
	Outline of the program: 
				<Program code for the examination of multiple person data sets (Check of the data base), 
				 descriptive analyses> 
	 
	
	Used variables (example dataset here: microcensus 2012): 
	Original variables: 
	   <EF1:   		Federal state 
	 	EF2:   		Administrative district 
	 	EF3:  		Number of selected district 
	 	EF4:  		Serial number of household in selected district 
	 	EF5:  		Serial number of person in the household
	 	EF25:  		Serial number of familiy in the household
	 	EF30:  		Population at primary residence
	 	EF31:  		Population in private households
	 	EF44:  		Age
	 	EF46:  		Sex
	 	EF49:  		Marital status
	 	EF310: 		Highest school-leaving qualification
	 	EF952: 		Standard projection factor year>
	
	 
	Newly created variables in this program code <syntax_name.do>: 
	   <marr:       dichotomous variable married yes/no
		persnr:  	Number of person
	 	famnr:   	Number of familiy 
	 	hhnr:    	Number of household 
	 	nrdiff:  	Difference number of household - number of family
	 	piddiff: 	Test unique number of person>
	
	Note: If you use variables that were created in a previous program code, please list them in a separate block. 
	
	Variable projection factor: EF952
	
	
*/
*	-------------------------------------------------------------------------------------------------------------------



*	-------------------------------------------------------------------------------------------------------------------
*	1. Editing data	-------------------------------------------------------------------------------------------------------------------


*	a. Input dataset
*	Memory space- and runtime-efficient direct selection of necessary variables

*	Population in private households (EF31 = 1) at the place of permanent residence (EF30 < 3)

use EF1 - EF5s EF25 EF30 EF31 EF44 EF46 EF49 EF310 EF952 ///
using "$datenpfad/$dateiname" if EF31 == 1 &  EF30 < 3, clear 

*	Create temporary work dataset

save "$neudatenpfad/TEMP1.dta", replace // This dataset is no longer needed after the analysis and can be deleted at the end.
                                      // Better than deletion: don't even save temporary datasets. 


*	b. Generate dichotomous variable marr which shows if a person is married. 

recode EF49 (2 = 1) (1 3 4 5 6 7 = 0), gen (marr)

* 	Assistance for microcensus users
*	c. Generate number of household and family from EF1, EF2, EF3, EF4, EF5, EF25 // 
*	persnr: EF1 to EF5
*	hh: EF1 to EF4s
*	famnr: EF1 to EF4s + EF25

generate double persnr=(EF1*10000000000) + (EF2*1000000000) + (EF3*100000) + (EF4*1000) + (EF5*10)
generate double hhnr=(EF1*10000000000) + (EF2*1000000000) + (EF3*100000) + (EF4*1000) + 1
generate double famnr=(EF1*10000000000) + (EF2*1000000000) + (EF3*100000) + (EF4*1000) + (EF25*1)
format %13.0g persnr hhnr famnr


*	d. Preparation for test: number of household unequal number of family
*	If: number of family (famnr) unequal number of household (hhnr)
*	Then: difference number of household (hhnr) - number of family (nrdiff) equals 1

generate nrdiff = 0

replace nrdiff = 1 if famnr ~= hhnr


*	e. Preparation test: multiple number of person
*	If number of person (persnr) unequal predecessor then test unique number of person (piddiff) equals 1 

sort persnr

generate piddiff = 0
replace piddiff = 1 if persnr == persnr[_n-1]


*	f. Labeling variables and values

label variable marr     "Married"

label variable hhnr 	"Number of household"
label variable famnr 	"Number of family"
label variable persnr 	"Number of person"

label variable piddiff 	"Test unique number of person"
label variable nrdiff 	"Difference number of household - number of family"

label define marr       1 "yes" ///
                        0 "no"
label values marr marr
						
label define nrdiffer 	1 "Number of household and number of family different" ///
						0 "Number of household and number of family identical"
label values nrdiff nrdiffer

label define piddiffer 	1 "Number of person multiple" ///
						0 "Number of person unique"
label values piddiff piddiffer

*	create temporary dataset

save "$neudatenpfad/TEMP2.dta", replace // This dataset is no longer needed after the analysis and can be deleted at the end.

	
	
*	-------------------------------------------------------------------------------------------------------------------
*	2. Data analyses	-------------------------------------------------------------------------------------------------------------------

/*
	For every generated result it has to be stated if it is to be checked for confidentiality or if it is generated 
	solely for the purpose of the RDC output checking process. This declaration can be made as a general comment 
	such as "All produced results are needed." or "Only weighted results are needed.", as long as the wording is 
	unambiguous.
 
*/

*	Data analyses full dataset

*	Note for the RDC: the results generated under point 2 are not to be released.

*	Output no. 2.1: Number of household unequal to number of family

tabulate nrdiff


*	Output no. 2.2: Test multiple number of person

tabulate piddiff


*	Data analyses by variable marr

keep if marr == 1


*	Output no. 2.3: Number of household unequal number of family

tabulate nrdiff


*	Output no. 2.4: Test multiple number of person

tabulate piddiff


*	TEMP2.dta aufrufen
use "$neudatenpfad\TEMP2.dta", clear



*	-------------------------------------------------------------------------------------------------------------------
*	3. Weighted analyses	-------------------------------------------------------------------------------------------------------------------

/*	
	It is necessary for the check for confidentiality to always add the unweighted results when weighted 
	results are generated (see brochure 2.7 Output of the underlying numbers of cases and marking of the relations).

*/

*	Output no. 3.1: unweighted frequency of the federal state

tabulate EF1


*	Output no. 3.2: weighted frequency of the federal state

tabulate EF1 [iw = EF952]


*	Output no. 3.3: unweighted regression using the dependent variable age (EF44) and the independent variables
*	sex (EF46) and married (marr)

* Alternative a: Program aborts when an error occurs
reg EF44 EF46 verh

* Alternative b: Because of the "capture noisily" command, the program would not abort when an error occurs in the following line.
cap noi reg EF44 EF46 marr


*	Output no. 3.4: unweighted regression using the dependent variable age (EF44) and the independent variables
*	                sex (EF46) and married (marr)

reg EF44 EF46 marr [aweight = EF952]



*	-------------------------------------------------------------------------------------------------------------------
*	4. Analyses using filter conditions	-------------------------------------------------------------------------------------------------------------------

/*
	Remaining samples that represent the difference to the "total" value are to be avoided.
	Example: You generate a table "total" and a table "male".
	The remaining sample/difference is "female". It has to be generated as well (see brochure 2.9 Output of 
	difference groups and marking of relations and 3.2.4 Analysis of subgroups).
*/

*	Output no. 4.1: Frequency of the highest qualification by total

tabulate EF310


*	Output no. 4.2: Frequency of the highest qualification by male

tabulate EF310 if EF46 == 1


*	Output no. 4.3: Frequency of the highest qualification by female

tabulate EF310 if EF46 == 2 

/*
	Note: Alternatively, a cross table without a previous case selection can be used for this kind of analysis. 
	By generating a cross table, remaining samples are avoided directly.
*/



*	-------------------------------------------------------------------------------------------------------------------
*	5. Output of descriptive coefficients	-------------------------------------------------------------------------------------------------------------------

/*
	For reasons of confidentiality, minima and maxima of metric variables are to be suppressed (see brochure 3.2.1
	Output of individual values). 
	The use of the tabulate-command is to be avoided when outputting descriptive key figures. Please use the 
	tabstat command.
*/

*	Output no. 5.1: Number of cases, mean and standard deviation for age of all single persons (EF49==1)

*	Option 1: Use of the tabstat-command

tabstat EF44 if EF49 == 1, statistics(n mean sd) // ATTENTION: Filter --> output remaining subgroups

*	Output remaining subgroup
tabstat EF44 if EF49 != 1, statistics(n mean sd)

/*
	Option 2: Use of the sum-command
	When using the sum-command, it is necessary to rewrite it. The use of the option detail is to be avoided. 
	The ado-file "makematrix.ado" has to be loaded in advance.
*/

makematrix, from(r(N) r(mean) r(sd)): summarize EF44 if EF49 == 1 // ATTENTION: Filter --> output remaining subgroups

*	Output remaining subgroup
makematrix, from(r(N) r(mean) r(sd)): summarize EF44 if EF49 != 1


/*
	Note: For economic statistics, when generating descriptive coefficients the largest and next to largest value 
	and the sum of all values of the concerned variable (e.g. volume of sales, income) are to be stated (see brochure
	3.1.3 Rules of dominance).
*/

*	-------------------------------------------------------------------------------------------------------------------
*	6. Output of dummy variables 	-------------------------------------------------------------------------------------------------------------------

/*
	In addition to the sum-command, a frequency table is to be produced because the number of cases is needed 
	for the check for confidentiality.
*/
 
*	Output no. 6.1:  Number of cases, mean and standard deviation for sex of all persons aged older than 65 (EF44 > 65)

summarize EF46 if EF44 > 65
*	for the check for confidentiality  
summarize EF46 if EF44 <= 65


*	Output no. 6.2  Frequency table for sex of all persons aged older than 65 (EF44 > 65)

tabulate EF46 if EF44 > 65

*	for the check for confidentiality 
tabulate EF46 if EF44 <= 65



*	-------------------------------------------------------------------------------------------------------------------
*	7. Output of value tables 	-------------------------------------------------------------------------------------------------------------------

/*
	When generating value tables, the corresponding cases of numbers always have to be stated as well.
	
	Generation of a user-defined table with number of cases, mean and valid frequency (N) of age (EF44) by 
	class of marital status (EF49)
*/

*	Output no. 7.1: Cross table age and marital status

tabstat EF44, statistics(n mean count) by (EF49)


*	-------------------------------------------------------------------------------------------------------------------
*	----
*	----	Part III: Program closure
*	----
*	-------------------------------------------------------------------------------------------------------------------

*	delete temporary datasets
erase "$neudatenpfad\TEMP1.dta"
erase "$neudatenpfad\TEMP2.dta"

*	close log-file
log close

exit

*	-------------------------------------------------------------------------------------------------------------------
*	N O T E:
/* 
	The sample program code at hand illustrates an example for an initial program code. If analyses are executed 
	in subsequent program codes and these analyses have a content-related connection to already generated results, 
	these connections are to be marked with a comment when processing the data and when the analysis is being
	commanded.
*/
*	-------------------------------------------------------------------------------------------------------------------





